class DBGraphParameters:
    def __init__(self):
        self.main_node_name = None
        self.main_rels_name = None
        self.secondary_node_name = None
        self.secondary_rels_name = None
        self.weight = None
        self.city_name = None
        self.node_geometry_identity = None
